//=============================================================================
//
//  CLASS CSurfacePlane - IMPLEMENTATION
//
//=============================================================================
#define RAYTRACER_CHECKERBOARD_CPP

//== INCLUDES =================================================================
#include "Checkerboard.h"


//== IMPLEMENTATION ==========================================================
void Checkerboard::renderGL() const
{

    //Generate plane
    glPushMatrix();
    if( m_selected )
    {
        GLfloat mat_ambient[] = { 1, 1, 1 };
        glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient );
    }
    else
        m_shader->shadeGL();

    VectorType3 nx( 1, 0, 0 );
    VectorType3 nz( 0, 0, 1 );

    //Rotation to xy-plane
    float       dx = dot( nz, m_v3Normal ) / m_v3Normal.length();
    float       dz = dot( nx, m_v3Normal ) / m_v3Normal.length();
    dx = 90 - 180 * acos( dx ) / M_PI;
    dz = 90 - 180 * acos( dz ) / M_PI;
    glRotatef( dx, 1, 0, 0 );
    glRotatef( dz, 0, 0, 1 );
    glTranslatef( m_v3Point[0], m_v3Point[1], m_v3Point[2] );

	//glDisable( GL_LIGHTING );

	//glBegin( GL_QUADS );    // rendering the plane quad. Note, it should be big

    // enough to cover all clip edge area.
    /*glVertex3f( -100, 0, -100 );
    glVertex3f( -100, 0, 100 );
    glVertex3f( 100, 0, 100 );
    glVertex3f( 100, 0, -100 );
    glNormal3f( m_v3Normal[0], m_v3Normal[1], m_v3Normal[2] );*/
	//glEnd();
	

	glColor3f( 1, 1, 1 );
	glBegin( GL_QUADS );
	float lengx = (m_v3X-m_v3Point).length();
	float lengy = (m_v3Y-m_v3Point).length();

	for ( int i = -100; i < 100; i++ )
	{
		for ( int j = -100; j < 100; j++ )
		{
			for ( int k = 0; k < m_xRaster; k++ )
			{
				for ( int l = 0; l < m_yRaster; l++ )
				{
					if ( k % 2 )
					{
						if ( l % 2 )
							m_shader->shadeGL();
						else
							m_shader2->shadeGL();

					}
					else
					{
						if ( l % 2 )
							m_shader2->shadeGL();
						else
							m_shader->shadeGL();
					}
					float x1 = ( i + ( k / (float) m_xRaster) ) * lengx;
					float z1 = ( j + ( l / (float) m_yRaster) ) * lengy;
					float x2 = ( i + ( (k+1) / (float) m_xRaster) ) * lengx;
					float z2 = ( j + ( (l+1) / (float) m_yRaster) ) * lengy;
					
					glVertex3f(  x1, 0, z1 );
					glVertex3f(  x1, 0, z2 );
					glVertex3f(  x2, 0, z2 );
					glVertex3f(  x2, 0, z1 );
					glNormal3f( m_v3Normal[0], m_v3Normal[1], m_v3Normal[2] );
				}
			}

		}
	}

    glEnd();
    glPopMatrix();
	
	//glEnable( GL_LIGHTING );
}


//-----------------------------------------------------------------------------
bool Checkerboard::intersect(
    const Ray       &clRay, 
    double        t0,     
    double        t1,     
    TracingContext  &tContext )  const
{
	MatrixType33 m_transformation;

    double    f_denominator = m_v3Normal | clRay.getDir();
    if( fabs( f_denominator ) < FLT_EPSILON )
        return false;

    double    t = ( m_v3Normal | (m_v3Point - clRay.getOrigin()) ) /
        f_denominator;
    if( t > t0 && t < t1 )
    {
        tContext.m_t = t;
        tContext.m_v3HitPoint = clRay.evaluate( t );
        tContext.m_v3Normal = m_v3Normal;

		//Check wether or not Hitpoint in checkerboard 
		VectorType3 v3_in_plane = tContext.m_v3HitPoint - m_v3Point;
		VectorType2 v2Coord = VectorType2((m_v3X-m_v3Point)|v3_in_plane, (m_v3Y-m_v3Point)|v3_in_plane);

		/*if ( v2TexCoord[0] > 0 && v2TexCoord[0] < 1 &&
			v2TexCoord[1] > 0 && v2TexCoord[1] < 1 )*/
		{
			double rounded;
			modf( v2Coord[0] * m_xRaster, &rounded);
			int rd = rounded;
			if ( ( rd % 2 && v2Coord[0] > 0 ) || 
				(  ( ! ( rd % 2 ) ) && v2Coord[0] < 0 ))
			{
				modf( v2Coord[1] * m_yRaster, &rounded);
				rd = rounded;
				if ( ( rd % 2 && v2Coord[1] > 0 ) || 
				(  ( ! ( rd % 2 ) ) && v2Coord[1] < 0 ) )
					tContext.m_pclShader = m_shader;
				else
					tContext.m_pclShader = m_shader2;
			}
			else
			{
				modf( v2Coord[1] * m_yRaster, &rounded);
				rd = rounded;
				if ( ( ( (!( rd % 2 ) )&& v2Coord[1] > 0 ) || 
				(  ( ( rd % 2 ) ) && v2Coord[1] < 0 ) ) )
					tContext.m_pclShader = m_shader;
				else
					tContext.m_pclShader = m_shader2;
			}
			
			return true;
		}
		return false;
    }

    return false;
}



//rotate object
void Checkerboard::rotate(
    float   rotx,   
    float   /*roty*/,   
    float   rotz )  
{
    //rot y ohne Funktion bei Ebenen
    //Drehmatrix um x-Achse
    rotx = ( rotx ) * M_PI / 180;
    rotz = ( rotz ) * M_PI / 180;

    VectorType3     x1 = VectorType3( 1, 0, 0 );
    VectorType3     x2 = VectorType3( 0, cos( rotx ), -sin( rotx ) );
    VectorType3     x3 = VectorType3( 0, sin( rotx ), cos( rotx ) );
    MatrixType33    rx;
    rx.setRow( 0, x1 );
    rx.setRow( 1, x2 );
    rx.setRow( 2, x3 );

    VectorType3     nx( 1, 0, 0 );
    VectorType3     nz( 0, 0, 1 );
    VectorType3     z1 = VectorType3( cos( rotz ), -sin( rotz ), 0 );
    VectorType3     z2 = VectorType3( sin( rotz ), cos( rotz ), 0 );
    VectorType3     z3 = VectorType3( 0, 0, 1 );
    MatrixType33    rz;
    rz.setRow( 0, z1 );
    rz.setRow( 1, z2 );
    rz.setRow( 2, z3 );

    VectorType3 ny = VectorType3( 0, 1, 0 );
    m_v3Normal = rx * rz * ny;
}


void Checkerboard::getRotation(
    int *rotx,  
    int *roty,  
    int *rotz ) 
{
        VectorType3 nx( 1, 0, 0 );
    VectorType3 nz( 0, 0, 1 );

    //Rotation to xy-plane
    *rotx = 90 - 180 * acos( dot( nz, m_v3Normal ) / m_v3Normal.length() ) / M_PI;
    *rotz = 90 - 180 * acos( dot( nx, m_v3Normal ) / m_v3Normal.length() ) / M_PI;
    *roty = 0;
}
