//=============================================================================
//
//  CLASS PhongShader - IMPLEMENTATION
//
//=============================================================================
#define RAYTRACER_PHONGSHADER_CPP

//== INCLUDES =================================================================
#include "PhongShader.h"

//== IMPLEMENTATION ==========================================================
ColorType PhongShader::shade(
    const TracingContext    &tContext )  const
{

    /*
		Das empirische Blinn-Phong Beleuchtungsmodell.
		Die relevanten Felder im TracingContext sind:

		v3Outgoing        - ausgehende Strahlrichtung
		v3Incoming        - einfallende Strahlrichtung
		v3Normal          - Normalenvektor im zu schattierenden Punkt
		colLightIntensity - Farbe des einfallenden Lichtstrahls
	*/

		
	const VectorType3& v3Normal = tContext.m_v3Normal;
	double cosTheta = v3Normal | tContext.m_v3Incoming;
	if (cosTheta<0) return ColorType(0,0,0);

	VectorType3 v3Halfway = tContext.m_v3Incoming + tContext.m_v3Outgoing;
	v3Halfway.normalize();
	double highlight =  pow(v3Normal|v3Halfway, m_rShininess);
	  	
	return tContext.m_colLightIntensity*(m_colDiffuse*cosTheta+m_colSpecular*highlight);

    return 0;
};


void PhongShader::shadeGL() const
{
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    GLfloat mat_specular[] = { m_colSpecular.red(), m_colSpecular.green(), m_colSpecular.blue() };
    GLfloat mat_shininess = m_rShininess;
    GLfloat mat_ambient[] = { m_colAmbient.red(), m_colAmbient.green(), m_colAmbient.blue() };
    GLfloat mat_diffuse[] = { m_colDiffuse.red(), m_colDiffuse.green(), m_colDiffuse.blue() };
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular );
    glMaterialfv( GL_FRONT, GL_SHININESS, &mat_shininess );
    glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient );
    glMaterialfv( GL_FRONT, GL_DIFFUSE, mat_diffuse );
}
